@@ -1,4 +1,4 @@
-# Copyright 2010-2013 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2013-2020 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -15,6 +15,7 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
+# memoized
 # Adapted from the original:
 # http://code.activestate.com/recipes/577452/ (r1)
 # Original Copyright notice:
@@ -40,58 +41,61 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
- 
-from __future__ import unicode_literals
-from __future__ import print_function
 
-from functools import partial, wraps
+
 import inspect
 import warnings
 
-class Memoized(object):
-    def __init__(self, func):
-        self.func = func
-    def __get__(self, obj, objtype=None):
-        if obj is None:
-            return self.func
-        return wraps(self.func)(partial(self, obj))
-    def __call__(self, *args, **kw):
-        obj = args[0]
-        try:
-            cache = obj.__cache
-        except AttributeError:
-            cache = obj.__cache = {}
-        key = (self.func, args[1:], frozenset(kw.items()))
-        try:
-            res = cache[key]
-        except KeyError:
-            res = cache[key] = self.func(*args, **kw)
-        return res
+import platform
+import pkg_resources
+import wrapt
 
-memoized=Memoized
+@wrapt.decorator
+def memoized(wrapped, instance, args, kwargs):
+    cache_location = instance
+    if instance is None:
+        cache_location = wrapped
 
+    try:
+        cache = cache_location.__cache__
+    except AttributeError:
+        cache = cache_location.__cache__ = {}
 
-class Deprecated(object):
-    def __init__(self, message):
-        self.message = message
+    key = (wrapped, args[:], frozenset(kwargs.items()))
+    try:
+        res = cache[key]
+    except KeyError:
+        res = cache[key] = wrapped(*args, **kwargs)
+    return res
 
-    def __call__(self, something):
-        if inspect.isfunction(something):
-            func = something
-        elif inspect.isclass(something):
-            func = something.__init__
+def deprecated(message, version='n/a'):
+    def catch_wrapped(f):
+        def is_init_or_classmethod(member):
+            if inspect.ismethod(member) and member.__self__ is f:
+                return True
+            return (inspect.ismethod(member) or inspect.isfunction(member)) and member.__name__ == '__init__'
+
+        @wrapt.decorator
+        def deprecated_wrapper(wrapped, instance, args, kwargs):
+            if is_init_or_classmethod(wrapped):
+                deprecated_thing = wrapped.__self__.__class__ 
+                warnings.warn('DEPRECATED: %s. %s' % (deprecated_thing.__class__, message), DeprecationWarning, stacklevel=2)
+            else:
+                deprecated_thing = wrapped
+                warnings.warn('DEPRECATED: %s. %s' % (deprecated_thing, message), DeprecationWarning, stacklevel=2)
+                
+            return wrapped(*args, **kwargs)
+
+        if pkg_resources.parse_version(platform.python_version()) > pkg_resources.parse_version('3.6') and f.__doc__:
+            f.__doc__ = '%s\n\n.. deprecated:: %s\n   %s' % (f.__doc__, version, message)
+
+        if inspect.isclass(f):
+            for name, method in inspect.getmembers(f, predicate=is_init_or_classmethod):
+                setattr(f, name, deprecated_wrapper(method))
+            return f
         else:
-            raise AssertionError('@deprecated can only be used for classes, functions or methods')
+            return deprecated_wrapper(f)
 
-        @wraps(func)
-        def deprecated_wrapper(*args, **kwds):
-            warnings.warn('DEPRECATED: %s. %s' % (something, self.message), DeprecationWarning, stacklevel=2)
-            return func(*args, **kwds)
+    return catch_wrapped
 
-        if inspect.isfunction(something):
-            return deprecated_wrapper
-        elif inspect.isclass(something):
-            something.__init__ = deprecated_wrapper
-            return something
 
-deprecated=Deprecated
