@@ -1,4 +1,4 @@
-# Copyright 2013-2020 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2013-2021 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -243,6 +243,10 @@ class Dependency(object):
         except:
             return None
 
+    @property
+    def is_component(self):
+        return bool(self.distribution.get_entry_map().get('reahl.versions', {}))
+
 
 class Version(object):
     def __init__(self, egg, version_number_string):
@@ -398,7 +402,7 @@ class ReahlEgg:
         dir_or_egg_name = translations_entry_point.dist.location.split(os.sep)[-1]
         paths = [p for p in module.__path__ if p.find('%s%s' % (dir_or_egg_name, os.path.sep)) > 0]
         paths = [p for p in paths if not p.startswith(os.path.join(translations_entry_point.dist.location, '.'))]
-        unique_paths = [p.split('%s/' % dir_or_egg_name)[-1] for p in paths]
+        unique_paths = [p.split('%s%s' % (dir_or_egg_name, os.path.sep))[-1] for p in paths]
         unique_paths = set([p for p in unique_paths if '.egg' not in p])
         assert len(unique_paths) <= 1, \
             'Only one translations package per component is allowed, found %s for %s' % (paths, translations_entry_point.dist)
