@@ -1,4 +1,4 @@
-# Copyright 2010-2013 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2013-2020 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -16,39 +16,45 @@
 
 """Exceptions used throughout several Reahl components."""
 
-from __future__ import unicode_literals
-from __future__ import print_function
-import six
-import inspect
 import sys
 
-from decorator import decorator
+import wrapt
+from wrapt.wrappers import PartialCallableObjectProxy
+import inspect
+
+
+from collections.abc import Callable
+
 
-from reahl.component.i18n import Translator
-import collections
 
-_ = Translator('reahl-component')
 
 class DomainException(Exception):
     """Any exception indicating an application-specific error condition that 
        should be communicated to a user.
 
-       :param commit: Set to True to indicate that the current database transaction 
-                      should be committed. By default transactions are rolled back
-                      when a DomainException is raised.
-    """
-    def __init__(self, commit=False):
-        self.commit = commit
+       :keyword commit: Set to True to indicate that the current database transaction 
+                        should be committed. By default transactions are rolled back
+                        when a DomainException is raised.
+       :keyword message: Optional error message.
+       :keyword detail_messages: A list of error messages giving more detail about the exception.
 
-    __hash__ = None
-    def __eq__(self, other):
-        return isinstance(other, self.__class__) and self.commit == other.commit
+       .. versionchanged: 5.0
+          Added `detail_messages` kwarg.
+    """
+    def __init__(self, commit=False, message=None, detail_messages=[]):
+        super().__init__(message)
+        self.commit = commit
+        self.message = message
+        self.detail_messages = detail_messages
     
     def __reduce__(self):
-        return (self.__class__, (self.commit,))
+        return (self.__class__, (self.commit, self.message, self.detail_messages))
     
     def as_user_message(self):
-        return _('An error occurred: %s' % self.__class__.__name__)
+        # To ensure this module can be imported at a very low level
+        from reahl.component.i18n import Catalogue
+        _ = Catalogue('reahl-component')
+        return self.message if self.message else _('An error occurred: %s' % self.__class__.__name__)
 
 
 class AccessRestricted(Exception):
@@ -68,13 +74,13 @@ class IncorrectArgumentError(ProgrammerError):
         return '%s (%s)' % (self.explanation, self.cause)
 
 
-class NotYetAvailable(object):
+class NotYetAvailable:
     def __init__(self, name):
         self.name = name
     def __str__(self):
         return '<%s name=%s>' % (self.__class__.__name__, self.name)
 
-class DeferredImport(object):
+class DeferredImport:
     def __init__(self, value_or_string):
         self.value_or_string = value_or_string
 
@@ -96,7 +102,7 @@ class DeferredImport(object):
             raise ProgrammerError('Could not find %s in %s (from %s)' % (class_name, module_name, string_spec))
 
     def coerce_to_type(self, type_or_string):
-        if isinstance(type_or_string, six.string_types):
+        if isinstance(type_or_string, str):
             return self.import_string_spec(type_or_string)
         else:
             return type_or_string
@@ -112,7 +118,7 @@ class ArgumentCheck(Exception):
             else:
                 return
 
-        if not value:
+        if value is None:
             if not self.allow_none:
                 self.raise_with(func, name, value)
             else:
@@ -128,10 +134,10 @@ class ArgumentCheck(Exception):
         self.arg_name = arg_name
         self.value = value
         raise self
-    
+
 class TypeBasedArgumentCheck(ArgumentCheck):
     def __init__(self, type_or_string, allow_none=False):
-        super(TypeBasedArgumentCheck, self).__init__(allow_none=allow_none)
+        super().__init__(allow_none=allow_none)
         self.type_ = DeferredImport(type_or_string)
 
     def is_marked_with_attribute(self, value):
@@ -143,7 +149,7 @@ class IsInstance(TypeBasedArgumentCheck):
 
     def __str__(self):
         return '%s: %s should be an instance of %s (got %s instead)' % (self.func, self.arg_name, self.type_.value, self.value)
-        
+
 class IsSubclass(TypeBasedArgumentCheck):
     def is_valid(self, value):
         return inspect.isclass(value) and (issubclass(value, self.type_.value) or self.is_marked_with_attribute(value))
@@ -151,17 +157,18 @@ class IsSubclass(TypeBasedArgumentCheck):
     def __str__(self):
         return '%s: %s should be %s or subclass of it (got %s instead)' % (self.func, self.arg_name, self.type_.value, self.value)
 
+
 class IsCallable(ArgumentCheck):
     def __init__(self, args=(), kwargs={}, allow_none=False):
-        super(IsCallable, self).__init__(allow_none=allow_none)
+        super().__init__(allow_none=allow_none)
         self.args = args
         self.kwargs = kwargs
 
     def check(self, func, name, value):
-        super(IsCallable, self).check(func, name, value)
+        super().check(func, name, value)
         if value:
             message = '%s will be called with %s' % (value, self.formatted_message())
-            checkargs_explained(message, value, *self.args, **self.kwargs)
+            ArgumentCheckedCallable(value, explanation=message).checkargs(*self.args, **self.kwargs)
 
     def formatted_message(self):
         formatted_args = ','.join([i.name for i in self.args])
@@ -174,52 +181,64 @@ class IsCallable(ArgumentCheck):
         return '(%s)' % (','.join(formatted_all))
 
     def is_valid(self, value):
-        return isinstance(value, collections.Callable)
+        return isinstance(value, Callable)
 
     def __str__(self):
         return '%s: %s should be a callable object (got %s)' % (self.func, self.arg_name, self.value)
 
-def checkargs(method, *args, **kwargs):
-    arg_checks = getattr(method, 'arg_checks', {})
-    try:
-        if inspect.ismethod(method) or inspect.isfunction(method):
-            to_check = method
-        elif inspect.isclass(method):
-            to_check = method.__init__
-        elif isinstance(method, collections.Callable):
-            to_check = method.__call__
+class ArgumentCheckedCallable:
+    def __init__(self, target, explanation=None):
+        self.target = target
+        self.explanation = explanation
+
+    def __call__(self, *args, **kwargs):
+        self.checkargs(*args, **kwargs)
+        return self.target(*args, **kwargs)
+
+    def checkargs(self, *args, **kwargs):
+        if isinstance(self.target, PartialCallableObjectProxy):
+            to_check = self.target.__call__
+        elif inspect.ismethod(self.target):
+            to_check = self.target
+        elif inspect.isfunction(self.target):
+            to_check = self.target
+        elif inspect.isclass(self.target):
+            to_check = self.target.__init__
+            args = (NotYetAvailable('self'), )+args
+        elif isinstance(self.target, Callable):
+            to_check = self.target.__call__
         else:
-            raise ProgrammerError('%s was expected to be a callable object' % method)
+            raise ProgrammerError('%s was expected to be a callable object' % self.target)
 
-        if inspect.ismethod(to_check) and not to_check.__self__:
-            call_args = (NotYetAvailable('self'),)+args
+        try:
+            try:
+                bound_args = inspect.getcallargs(to_check, *args, **kwargs)
+            except TypeError as ex:
+                ex.args = (('%s: ' % self.target)+ex.args[0],) + ex.args[1:]
+                raise
+            args_to_check = getattr(to_check, 'arg_checks', {})
+            for arg_name, arg_check in args_to_check.items():
+                if arg_name in bound_args.keys():
+                    arg_check.check(self.target, arg_name, bound_args[arg_name])
+        except (TypeError, ArgumentCheck) as ex:
+            if self.explanation:
+                _, _, tb = sys.exc_info()
+                new_ex = IncorrectArgumentError(self.explanation, ex)
+                raise new_ex.with_traceback(tb)
+            else:
+                raise
+
+
+def arg_checks(**checks):
+    def catch_wrapped(f):
+        if inspect.ismethoddescriptor(f):
+            f.__func__.arg_checks = checks
         else:
-            call_args = args
-        bound_args = inspect.getcallargs(to_check, *call_args, **kwargs)
-    except TypeError as ex:
-        ex.args = (('%s: ' % method)+ex.args[0],) + ex.args[1:]
-        raise
-    for arg_name, arg_check in arg_checks.items():
-        if arg_name in bound_args.keys():
-            arg_check.check(method, arg_name, bound_args[arg_name])
-
-def checkargs_explained(explanation, method, *args, **kwargs):
-    try:
-        checkargs(method, *args, **kwargs)
-    except (TypeError, ArgumentCheck) as ex:
-        _, _, tb = sys.exc_info()
-        new_ex = IncorrectArgumentError(explanation, ex)
-        six.reraise(new_ex.__class__, new_ex, tb)
+            f.arg_checks = checks
+        @wrapt.decorator
+        def check_call(wrapped, instance, args, kwargs):
+            return ArgumentCheckedCallable(wrapped)(*args, **kwargs)
+        return check_call(f)
+    return catch_wrapped
 
 
-class arg_checks(object):
-    def __init__(self, **arg_checks):
-        self.arg_checks = arg_checks
-    
-    def __call__(self, f):
-        f.arg_checks = self.arg_checks
-        def check_call(func, *args, **kwargs):
-            checkargs(func, *args, **kwargs)
-            return func(*args, **kwargs)
-        return decorator(check_call, f)
-
